package com.brackeen.javagamebook.test;

import java.awt.*;
import javax.swing.ImageIcon;

import com.brackeen.javagamebook.graphics.ScreenManager;

/**
    Prosta klasa abstrakcyjna, wykorzystywana do testowania. 
    Klasy pochodne powinny implementowa metod draw().
*/
public abstract class GameCore {

    protected static final int FONT_SIZE = 24;

    private static final DisplayMode POSSIBLE_MODES[] = {
        new DisplayMode(800, 600, 32, 0),
        new DisplayMode(800, 600, 24, 0),
        new DisplayMode(800, 600, 16, 0),
        new DisplayMode(640, 480, 32, 0),
        new DisplayMode(640, 480, 24, 0),
        new DisplayMode(640, 480, 16, 0)
    };

    private boolean isRunning;
    protected ScreenManager screen;

    /**
        Sygnalizuje ptli gry, e czas koczy prac.
    */
    public void stop() {
        isRunning = false;
    }

    /**
        Wywouje init() i gameLoop().
    */
    public void run() {
        try {
            init();
            gameLoop();
        }
        finally {
            screen.restoreScreen();
            lazilyExit();
        }
    }
    
    /**
        Koczy dziaanie VM w wtku demona. Wtek-demon czeka dwie
        sekundy przed wywoaniem System.exit(0). Poniewa VM koczy
        dziaanie gdy pozostaj uruchomione jedynie wtki-demony, mamy
        pewno, e System.exit(0) zostanie wywoana tylko w
        ostatecznoci. Jest to konieczne, gdy uruchomiony jest system
        Java Sound.
     */
    public void lazilyExit() {
        Thread thread = new Thread() {
            public void run() {
               // Na pocztek poczekaj na zakoczenie dziaania maszyny wirtualnej.
                try {
                    Thread.sleep(2000);
                }
                catch (InterruptedException ex) { }
               // System nadal dziaa, wic wymuszamy zakoczenie pracy.
                System.exit(0);
            }
        };
        thread.setDaemon(true);
        thread.start();
    }

    /**
        Ustawia tryb penoekranowy i inicjuje obiekty.
    */
    public void init() {
        screen = new ScreenManager();
        DisplayMode displayMode =
            screen.findFirstCompatibleMode(POSSIBLE_MODES);
        screen.setFullScreen(displayMode);

        Window window = screen.getFullScreenWindow();
        window.setFont(new Font("Dialog", Font.PLAIN, FONT_SIZE));
        window.setBackground(Color.blue);
        window.setForeground(Color.white);

        isRunning = true;
    }

    public Image loadImage(String fileName) {
        return new ImageIcon(fileName).getImage();
    }

    /**
        Uruchamia ptl gry, dziaajc a do wywoania metody stop().
    */
    public void gameLoop() {
        long startTime = System.currentTimeMillis();
        long currTime = startTime;

        while (isRunning) {
            long elapsedTime =
                System.currentTimeMillis() - currTime;
            currTime += elapsedTime;

            // Aktualizacja:
            update(elapsedTime);

            // Rysowanie:
            Graphics2D g = screen.getGraphics();
            draw(g);
            g.dispose();
            screen.update();

            // Chwila przerwy:
            try {
                Thread.sleep(20);
            }
            catch (InterruptedException ex) { }
        }
    }

    /**
        Aktualizuje stan gry (animacji) na podstawie okresu czasu, 
        jaki min od uruchomienia programu.
    */
    public void update(long elapsedTime) {
        // Nic nie wykonuj.
    }

    /**
        Rysowanie na ekranie. Klasy pochodne musz dziedziczy t metod.
    */
    public abstract void draw(Graphics2D g);
}
